//==============================================================================
//! @file        ElgatoGameCapture.js
//! @bc ------------------------------------------------------------------------
//! @ec @brief   Elgato Game Capture library for custom web page overlays
//! @author      T.Schnitzler
//! @date        15-May-16 TS - Creation
//! @bc ------------------------------------------------------------------------
//! @ec  @par    Copyright
//! @n           (c) 2016, Elgato Systems GmbH. All Rights Reserved.
//! @n           
//! @n           Information in this file is the intellectual property of
//!              Elgato Systems GmbH, and contains trade secrets
//!              that must be stored and viewed confidentially.
//==============================================================================

(function (window) {

    // TS NOTE: Usage: Here is an example
    //          var egc = window.ElgatoGameCapture;
    //          egc.getCustomOverlaySetting("test", function (data) {
    //              alert(data);
    //          });

    // TS NOTE: Run Google Chrome with command-line parameters "--disable-web-security --user-data-dir" 
    //          for debugging purposes to disable cross origin policy.
	// RM NOTE: This will not work anymore, because we stopped allowing to load the json files. For debugging
	//			add debuging port in BrowserClient.cpp and use chrome with localhost:[debuggingport]
	//			Currently the debugging port is set as 11111 in Debug builds of GC

    // Enable strict mode
    "use strict";

    function getLibrary() {

        var _library = {};

        // Returns the current Game Capture application language
        _library.getApplicationLanguage = function () {
            // Get language from URL parameters
            var language = getUrlParameter("language");

            return language;
        }

		// Returns the value of a certain setting in the Overlay*.json file ("customSource")
        _library.getOverlaySetting = function (name, callback) {

            // Get overlay
            getSettings(function (settings) {

                // Forward "customSource" child value to callback
                if (settings && callback)
                    callback(settings[name]);

            });
        }
		
        // Returns the value of a certain setting of the customSource in the Overlay*.json file ("customSource/customJsonData")
        _library.getCustomOverlaySetting = function (name, callback) {

            // Get "customJsonData"
            this.getOverlaySetting("customJsonData", function (customJsonData) {

                // Parse "customJsonData"
                if (customJsonData) {
                    var customJson = JSON.parse(customJsonData);

                    // Forward value to callback
                    if (customJson && callback)
                        callback(customJson[name]);

                }
            });
        }

        // Returns the complete settings of the customSource in the Overlay*.json file ("customSource/customJsonData")
        _library.getCustomOverlaySettings = function (callback) {

            // Get "customJsonData"
            this.getOverlaySetting("customJsonData", function (customJsonData) {

                // Parse "customJsonData"
                if (customJsonData) {
                    var customJson = JSON.parse(customJsonData);

                    // Forward value to callback
                    if (customJson && callback)
                        callback(customJson);

                }
            });
        }

        var newCustomJson = {};

        // Set the complete settings of the customSource
        _library.setCustomOverlaySettings = function (settings) {
            newCustomJson = settings;
        }

        // Set the value of a certain setting of the customSource
        _library.setCustomOverlaySetting = function (name, data) {
            newCustomJson[name] = data;
        }

        // Save the settings of the customSource in the Overlay*.json file ("customSource/customJsonData")
        _library.saveCustomOverlaySettings = function () {
            location.hash = 'settings=' + JSON.stringify(newCustomJson);
        }

        // --------------------------------------------------------------------------------
        // Helper
        // --------------------------------------------------------------------------------

        // Get the value of a certain parameter contained in the URL
        function getUrlParameter(name) {

            // Get URL
            var url = window.location.href;
            name = name.replace(/[\[\]]/g, "\\$&");

            // Find parameter by name and return its value
            var regEx = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)", "i");
            var results = regEx.exec(url);
            if (!results)
                return null;
            if (!results[2])
                return null;
            return decodeURIComponent(results[2].replace(/\+/g, " "));
        }

		// Read custom source JSON from file
        function getJSON(dataCallback) {
            // Prepare HTTP GET request
            var xmlHttp = new XMLHttpRequest();
            xmlHttp.onreadystatechange = function () {
                if (xmlHttp.readyState == xmlHttp.DONE) {
                    // Forward data to callback
                    if (dataCallback)
                        dataCallback(xmlHttp.responseText);
                }
            };

            // Send HTPP GET request
            xmlHttp.open("GET", "http://egc/settings", true);
            xmlHttp.send();
        }

		// gets the custom source
		function getSettings(callback) 
		{
			getJSON(function (jsonData) 
			{
				var json = JSON.parse(jsonData);
				if (callback)
				callback(json);
			});
		}

        return _library;
    }

    // Make library globally accessible
    if (typeof (window.ElgatoGameCapture) === "undefined") {
        window.ElgatoGameCapture = getLibrary();
    }

})(window);
